//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
// Code generated by Alibaba Cloud SDK Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package ali

import (
	pbpod "code.gitlink.org.cn/JCCE/PCM.git/adaptor/pcm_pod/gen/idl"
	"code.gitlink.org.cn/JCCE/PCM.git/adaptor/pcm_pod/server"
	"code.gitlink.org.cn/JCCE/PCM.git/common/tenanter"
	"errors"
	"flag"
	"github.com/aliyun/alibaba-cloud-sdk-go/sdk/requests"
	"github.com/aliyun/alibaba-cloud-sdk-go/sdk/responses"
	"github.com/golang/glog"
)

// UpdateContainerGroup invokes the eci.UpdateContainerGroup API synchronously
// api document: https://help.aliyun.com/api/eci/updatecontainergroup.html
func UpdateContainerGroup(request *UpdateContainerGroupRequest) (response *UpdateContainerGroupResponse, err error) {

	provider := pbpod.CloudProvider(request.ProviderId)
	var configFile string
	flag.StringVar(&configFile, "confAli", "configs/tenanter.yaml", "tenanter.yaml")
	flag.Parse()
	defer glog.Flush()

	if err := tenanter.LoadCloudConfigsFromFile(configFile); err != nil {
		if !errors.Is(err, tenanter.ErrLoadTenanterFileEmpty) {
			glog.Fatalf("tenanter.LoadCloudConfigsFromFile error %+v", err)
		}
		glog.Warningf("tenanter.LoadCloudConfigsFromFile empty file path %s", configFile)
	}

	glog.Infof("load tenant from file finished")
	var regionId int32
	switch request.ProviderId {
	case 0:
		regionId, _ = tenanter.GetAliRegionId(request.RegionId)
	case 1:
		regionId, _ = tenanter.GetTencentRegionId(request.RegionId)
	case 2:
		regionId, _ = tenanter.GetHuaweiRegionId(request.RegionId)
	case 3:
		regionId, _ = tenanter.GetK8SRegionId(request.RegionId)
	}
	containers := *request.Container

	requestPCM := &pbpod.UpdatePodReq{
		RequestSource:  "ali",
		Provider:       pbpod.CloudProvider(provider),
		AccountName:    request.AccountName,
		PcmId:          request.PcmId,
		PodId:          request.ContainerGroupId,
		PodName:        request.ContainerGroupName,
		Namespace:      request.Namespace,
		RegionId:       regionId,
		ContainerImage: containers[0].Image,
		ContainerName:  containers[0].Name,
		CpuPod:         string(containers[0].Cpu),
		MemoryPod:      string(containers[0].Memory),
		RestartPolicy:  request.RestartPolicy,
		Labels:         "sss",
	}

	resp, err := server.UpdatePod(nil, requestPCM)

	response = &UpdateContainerGroupResponse{
		BaseResponse: nil,
		RequestId:    resp.RequestId,
	}

	return response, err
}

// UpdateContainerGroupRequest is the request struct for api UpdateContainerGroup
type UpdateContainerGroupRequest struct {
	*requests.RpcRequest
	/*********PCM param************/
	RequestSource      string `position:"Query" name:"RequestSource"`
	ProviderId         int32  `position:"Query" name:"ProviderId"`
	AccountName        string `position:"Query" name:"AccountName"`
	Namespace          string `position:"Query" name:"Namespace"`
	PcmId              string `position:"Query" name:"PcmId"`
	PodName            string `position:"Query" name:"PodName"`
	ContainerGroupName string `position:"Query" name:"ContainerGroupName"`
	/*********PCM param************/
	OwnerId                 requests.Integer                               `position:"Query" name:"OwnerId"`
	ResourceOwnerAccount    string                                         `position:"Query" name:"ResourceOwnerAccount"`
	ResourceOwnerId         requests.Integer                               `position:"Query" name:"ResourceOwnerId"`
	OwnerAccount            string                                         `position:"Query" name:"OwnerAccount"`
	RegionId                string                                         `position:"Query" name:"RegionId"`
	ContainerGroupId        string                                         `position:"Query" name:"ContainerGroupId"`
	RestartPolicy           string                                         `position:"Query" name:"RestartPolicy"`
	Tag                     *[]UpdateContainerGroupTag                     `position:"Query" name:"Tag" type:"Repeated"`
	Volume                  *[]UpdateContainerGroupVolume                  `position:"Query" name:"Volume" type:"Repeated"`
	Container               *[]UpdateContainerGroupContainer               `position:"Query" name:"Container" type:"Repeated"`
	InitContainer           *[]UpdateContainerGroupInitContainer           `position:"Query" name:"InitContainer" type:"Repeated"`
	ImageRegistryCredential *[]UpdateContainerGroupImageRegistryCredential `position:"Query" name:"ImageRegistryCredential" type:"Repeated"`
	ClientToken             string                                         `position:"Query" name:"ClientToken"`
	Cpu                     requests.Float                                 `position:"Query" name:"Cpu"`
	Memory                  requests.Float                                 `position:"Query" name:"Memory"`
	DnsConfig               UpdateContainerGroupDnsConfig                  `position:"Query" name:"DnsConfig" type:"Struct"`
}

type UpdateContainerGroupTag struct {
	Key   string `name:"Key"`
	Value string `name:"Value"`
}

type UpdateContainerGroupVolume struct {
	Name             string                               `name:"Name"`
	Type             string                               `name:"Type"`
	NFSVolume        UpdateContainerGroupNFSVolume        `name:"NFSVolume" type:"Struct"`
	ConfigFileVolume UpdateContainerGroupConfigFileVolume `name:"ConfigFileVolume" type:"Struct"`
	EmptyDirVolume   UpdateContainerGroupEmptyDirVolume   `name:"EmptyDirVolume" type:"Struct"`
}

type UpdateContainerGroupContainer struct {
	Name            string                              `name:"Name"`
	Image           string                              `name:"Image"`
	Cpu             requests.Float                      `name:"Cpu"`
	Memory          requests.Float                      `name:"Memory"`
	WorkingDir      string                              `name:"WorkingDir"`
	ImagePullPolicy string                              `name:"ImagePullPolicy"`
	Stdin           requests.Boolean                    `name:"Stdin"`
	StdinOnce       requests.Boolean                    `name:"StdinOnce"`
	Tty             requests.Boolean                    `name:"Tty"`
	Command         []string                            `name:"Command" type:"Repeated"`
	Arg             []string                            `name:"Arg" type:"Repeated"`
	EnvironmentVar  []string                            `name:"EnvironmentVar" type:"Repeated"`
	Port            []string                            `name:"Port" type:"Repeated"`
	VolumeMount     []string                            `name:"VolumeMount" type:"Repeated"`
	Gpu             requests.Integer                    `name:"Gpu"`
	ReadinessProbe  UpdateContainerGroupReadinessProbe  `name:"ReadinessProbe" type:"Struct"`
	LivenessProbe   UpdateContainerGroupLivenessProbe   `name:"LivenessProbe" type:"Struct"`
	SecurityContext UpdateContainerGroupSecurityContext `name:"SecurityContext" type:"Struct"`
}

type UpdateContainerGroupInitContainer struct {
	Name            string                                `name:"Name"`
	Image           string                                `name:"Image"`
	Cpu             requests.Float                        `name:"Cpu"`
	Memory          requests.Float                        `name:"Memory"`
	WorkingDir      string                                `name:"WorkingDir"`
	ImagePullPolicy string                                `name:"ImagePullPolicy"`
	Stdin           requests.Boolean                      `name:"Stdin"`
	StdinOnce       requests.Boolean                      `name:"StdinOnce"`
	Tty             requests.Boolean                      `name:"Tty"`
	Command         []string                              `name:"Command" type:"Repeated"`
	Arg             []string                              `name:"Arg" type:"Repeated"`
	EnvironmentVar  *[]UpdateContainerGroupEnvironmentVar `name:"EnvironmentVar" type:"Repeated"`
	Port            *[]UpdateContainerGroupPort           `name:"Port" type:"Repeated"`
	VolumeMount     *[]UpdateContainerGroupVolumeMount    `name:"VolumeMount" type:"Repeated"`
	Gpu             requests.Integer                      `name:"Gpu"`
	SecurityContext UpdateContainerGroupSecurityContext   `name:"SecurityContext" type:"Struct"`
}

type UpdateContainerGroupImageRegistryCredential struct {
	Server   string `name:"Server"`
	UserName string `name:"UserName"`
	Password string `name:"Password"`
}

type UpdateContainerGroupDnsConfig struct {
	NameServer []string                      `name:"NameServer"`
	Search     []string                      `name:"Search"`
	Option     *[]UpdateContainerGroupOption `name:"Option"`
}

type UpdateContainerGroupNFSVolume struct {
	Server   string           `name:"Server"`
	Path     string           `name:"Path"`
	ReadOnly requests.Boolean `name:"ReadOnly"`
}

type UpdateContainerGroupConfigFileVolume struct {
	ConfigFileToPath *[]UpdateContainerGroupConfigFileToPath `name:"ConfigFileToPath"`
}

type UpdateContainerGroupConfigFileToPath struct {
	Content string `name:"Content"`
	Path    string `name:"Path"`
}

type UpdateContainerGroupEmptyDirVolume struct {
	Medium string `name:"Medium"`
}

type UpdateContainerGroupReadinessProbe struct {
	InitialDelaySeconds requests.Integer              `name:"InitialDelaySeconds"`
	PeriodSeconds       requests.Integer              `name:"PeriodSeconds"`
	SuccessThreshold    requests.Integer              `name:"SuccessThreshold"`
	FailureThreshold    requests.Integer              `name:"FailureThreshold"`
	TimeoutSeconds      requests.Integer              `name:"TimeoutSeconds"`
	TcpSocket           UpdateContainerGroupTcpSocket `name:"TcpSocket"`
	Exec                UpdateContainerGroupExec      `name:"Exec"`
	HttpGet             UpdateContainerGroupHttpGet   `name:"HttpGet"`
}

type UpdateContainerGroupTcpSocket struct {
	Port requests.Integer `name:"Port"`
}

type UpdateContainerGroupExec struct {
	Command []string `name:"Command"`
}

type UpdateContainerGroupHttpGet struct {
	Path   string           `name:"Path"`
	Port   requests.Integer `name:"Port"`
	Scheme string           `name:"Scheme"`
}

type UpdateContainerGroupLivenessProbe struct {
	InitialDelaySeconds requests.Integer              `name:"InitialDelaySeconds"`
	PeriodSeconds       requests.Integer              `name:"PeriodSeconds"`
	SuccessThreshold    requests.Integer              `name:"SuccessThreshold"`
	FailureThreshold    requests.Integer              `name:"FailureThreshold"`
	TimeoutSeconds      requests.Integer              `name:"TimeoutSeconds"`
	TcpSocket           UpdateContainerGroupTcpSocket `name:"TcpSocket"`
	Exec                UpdateContainerGroupExec      `name:"Exec"`
	HttpGet             UpdateContainerGroupHttpGet   `name:"HttpGet"`
}

type UpdateContainerGroupSecurityContext struct {
	ReadOnlyRootFilesystem requests.Boolean               `name:"ReadOnlyRootFilesystem"`
	RunAsUser              requests.Integer               `name:"RunAsUser"`
	Capability             UpdateContainerGroupCapability `name:"Capability"`
}

type UpdateContainerGroupCapability struct {
	Add []string `name:"Add"`
}

type UpdateContainerGroupEnvironmentVar struct {
	Key   string `name:"Key"`
	Value string `name:"Value"`
}

type UpdateContainerGroupPort struct {
	Port     requests.Integer `name:"Port"`
	Protocol string           `name:"Protocol"`
}

type UpdateContainerGroupVolumeMount struct {
	Name      string           `name:"Name"`
	MountPath string           `name:"MountPath"`
	SubPath   string           `name:"SubPath"`
	ReadOnly  requests.Boolean `name:"ReadOnly"`
}

type UpdateContainerGroupOption struct {
	Name  string `name:"Name"`
	Value string `name:"Value"`
}

// UpdateContainerGroupResponse is the response struct for api UpdateContainerGroup
type UpdateContainerGroupResponse struct {
	*responses.BaseResponse
	RequestId string `json:"RequestId" xml:"RequestId"`
}

// CreateUpdateContainerGroupRequest creates a request to invoke UpdateContainerGroup API
func CreateUpdateContainerGroupRequest() (request *UpdateContainerGroupRequest) {
	request = &UpdateContainerGroupRequest{
		RpcRequest: &requests.RpcRequest{},
	}
	request.InitWithApiInfo("Eci", "2018-08-08", "UpdateContainerGroup", "eci", "openAPI")
	return
}

// CreateUpdateContainerGroupResponse creates a response to parse from UpdateContainerGroup response
func CreateUpdateContainerGroupResponse() (response *UpdateContainerGroupResponse) {
	response = &UpdateContainerGroupResponse{
		BaseResponse: &responses.BaseResponse{},
	}
	return
}
