package server

import (
	"code.gitlink.org.cn/JCCE/PCM.git/adaptor/pcm_pod/gen/idl"
	"context"

	"github.com/golang/glog"
	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/status"
)

type Server struct {
	podpb.UnimplementedPodServiceServer
}

// GetProdRegions get available region for product
func (s *Server) GetProdRegions(ctx context.Context, req *podpb.GetPodRegionReq) (*podpb.GetPodRegionResp, error) {
	resp, err := GetPodRegion(ctx, req)
	if err != nil {
		glog.Errorf("CreatePods error %+v", err)
		return nil, status.Errorf(codes.Internal, err.Error())
	}
	return resp, nil
}

// CreatePods create multiple pcm_pod on multiple clouds
func (s *Server) CreatePods(ctx context.Context, req *podpb.CreatePodsReq) (*podpb.CreatePodsResp, error) {
	resp, err := CreatePods(ctx, req)
	if err != nil {
		glog.Errorf("CreatePods error %+v", err)
		return nil, status.Errorf(codes.Internal, err.Error())
	}
	return resp, nil
}

// CreatePod create pcm_pod on one cloud
func (s *Server) CreatePod(ctx context.Context, req *podpb.CreatePodReq) (*podpb.CreatePodResp, error) {
	resp, err := CreatePod(ctx, req)
	if err != nil {
		glog.Errorf("CreatePod error %+v", err)
		return nil, status.Errorf(codes.Internal, err.Error())
	}
	return resp, nil
}

// DeletePod delete specified pcm_pod
func (s *Server) DeletePod(ctx context.Context, req *podpb.DeletePodReq) (*podpb.DeletePodResp, error) {
	resp, err := DeletePod(ctx, req)
	if err != nil {
		glog.Errorf("DeletePod error %+v", err)
		return nil, status.Errorf(codes.Internal, err.Error())
	}
	return resp, nil
}

// UpdatePod update specified pcm_pod
func (s *Server) UpdatePod(ctx context.Context, req *podpb.UpdatePodReq) (*podpb.UpdatePodResp, error) {
	resp, err := UpdatePod(ctx, req)
	if err != nil {
		glog.Errorf("UpdatePod error %+v", err)
		return nil, status.Errorf(codes.Internal, err.Error())
	}
	return resp, nil
}

func (s *Server) ListPodDetail(ctx context.Context, req *podpb.ListPodDetailReq) (*podpb.ListPodDetailResp, error) {
	resp, err := ListPodDetail(ctx, req)
	if err != nil {
		glog.Errorf("ListPodDetail error %+v", err)
		return nil, status.Errorf(codes.Internal, err.Error())
	}
	return resp, nil
}

func (s *Server) ListPod(ctx context.Context, req *podpb.ListPodReq) (*podpb.ListPodResp, error) {
	resp, err := ListPod(ctx, req)
	if err != nil {
		glog.Errorf("ListPod error %+v", err)
		return nil, status.Errorf(codes.Internal, err.Error())
	}
	return resp, nil
}

func (s *Server) ListPodAll(ctx context.Context, req *podpb.ListPodAllReq) (*podpb.ListPodResp, error) {
	resp, err := ListPodAll(ctx)
	if err != nil {
		glog.Errorf("ListPodAll error %+v", err)
		return nil, status.Errorf(codes.Internal, err.Error())
	}
	return resp, nil
}
