package tianhe

/*
//#cgo LDFLAGS: -lslurm
#include<stdlib.h>
#include<slurm/slurm.h>
#include<slurm/slurm_errno.h>

 struct node_info_msg *get_node_info(){
   	struct node_info_msg* node_buffer;
       if(slurm_load_node ((time_t) NULL,
		&node_buffer, SHOW_ALL))
		return NULL;
	return node_buffer;
 }
  struct node_info_msg *get_single_node_info(char* name){
   	struct node_info_msg* node_buffer;
	if( slurm_load_node_single (&node_buffer, name, SHOW_DETAIL))
		return NULL;
	return node_buffer;
 }

 struct node_info* node_from_list(struct node_info_msg *list, int i){
	 return &list->node_array[i];
}
 void free_node_buffer(void* buffer){

	slurm_free_node_info_msg ((struct node_info_msg*)buffer);
  }


*/
import "C"
import (
	pbslurm "code.gitlink.org.cn/JCCE/PCM.git/adaptor/pcm_slurm/gen/idl"
)

type NodeInfoMsg struct {
	LastUpdate   int64
	RecordCount  uint32
	ErrorCode    uint32
	NodeInfoList []pbslurm.NodeInfo
}

func NodeInfoConvertCToGo(cStruct *C.struct_node_info) pbslurm.NodeInfo {
	var goStruct pbslurm.NodeInfo

	goStruct.Arch = C.GoString(cStruct.arch)
	goStruct.Boards = uint32(cStruct.boards)
	goStruct.BootTime = int64(cStruct.boot_time)
	goStruct.Cores = uint32(cStruct.cores)
	goStruct.CpuLoad = uint32(cStruct.cpu_load)
	goStruct.Cpus = uint32(cStruct.cpus)
	goStruct.Features = C.GoString(cStruct.features)
	goStruct.Gres = C.GoString(cStruct.gres)
	goStruct.Name = C.GoString(cStruct.name)
	goStruct.NodeAddr = C.GoString(cStruct.node_addr)
	goStruct.NodeHostname = C.GoString(cStruct.node_hostname)
	goStruct.NodeState = uint32(cStruct.node_state)
	goStruct.Os = C.GoString(cStruct.os)
	goStruct.RealMemory = uint64(cStruct.real_memory)
	goStruct.Reason = C.GoString(cStruct.reason)
	goStruct.ReasonTime = int64(cStruct.reason_time)
	goStruct.ReasonUid = uint32(cStruct.reason_uid)
	goStruct.SlurmdStartTime = int64(cStruct.slurmd_start_time)
	goStruct.Sockets = uint32(cStruct.sockets)
	goStruct.Threads = uint32(cStruct.threads)
	goStruct.TmpDisk = uint32(cStruct.tmp_disk)
	goStruct.Weight = uint32(cStruct.weight)
	return goStruct
}

func NodeDescriptorConvertCToGo(cStruct *C.struct_node_info) pbslurm.NodeInfo {
	var goStruct pbslurm.NodeInfo
	goStruct.Name = C.GoString(cStruct.name)
	return goStruct
}

func GetNodeInfo() NodeInfoMsg {
	var goNodeBuffer NodeInfoMsg
	cNodeBuffer := C.get_node_info()
	goNodeBuffer.RecordCount = uint32(cNodeBuffer.record_count)
	goNodeBuffer.NodeInfoList = make([]pbslurm.NodeInfo, cNodeBuffer.record_count, cNodeBuffer.record_count)
	for i := uint32(0); i < goNodeBuffer.RecordCount; i++ {
		Node := C.node_from_list(cNodeBuffer, C.int(i))
		goNode := NodeDescriptorConvertCToGo(Node)
		goNodeBuffer.NodeInfoList[i] = goNode
	}
	return goNodeBuffer
}

func Get_all_nodes() NodeInfoMsg {
	var go_node_buffer NodeInfoMsg
	c_node_buffer := C.get_node_info()
	if c_node_buffer == nil {
		go_node_buffer.LastUpdate = int64(0)
		go_node_buffer.RecordCount = uint32(0)
		go_node_buffer.ErrorCode = uint32(C.slurm_get_errno())
		return go_node_buffer
	}
	go_node_buffer.LastUpdate = int64(c_node_buffer.last_update)
	go_node_buffer.RecordCount = uint32(c_node_buffer.record_count)
	go_node_buffer.NodeInfoList = make([]pbslurm.NodeInfo, c_node_buffer.record_count, c_node_buffer.record_count)
	for i := uint32(0); i < go_node_buffer.RecordCount; i++ {
		node := C.node_from_list(c_node_buffer, C.int(i))
		go_node := NodeInfoConvertCToGo(node)
		go_node_buffer.NodeInfoList[i] = go_node
	}
	C.slurm_free_node_info_msg(c_node_buffer)

	return go_node_buffer
}
