package tianhe

/*
#cgo LDFLAGS: -lslurmdb

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <slurm/slurm.h>
#include <slurm/slurmdb.h>
#include <slurm/slurm_errno.h>
#include <memory.h>
#include <malloc.h>

typedef struct slurmdb_qos_rec {
	char *description;
	uint32_t id;
	uint32_t flags;
	uint32_t grace_time;
	uint64_t grp_cpu_mins;
	uint64_t grp_cpu_run_mins;
	uint32_t grp_cpus;
	uint32_t grp_jobs;
	uint32_t grp_mem;
	uint32_t grp_nodes;
	uint32_t grp_submit_jobs;
	uint32_t grp_wall;
	uint64_t max_cpu_mins_pj;
	uint64_t max_cpu_run_mins_pu;
	uint32_t max_cpus_pj;
	uint32_t max_cpus_pu;
	uint32_t max_jobs_pu;
	uint32_t max_nodes_pj;
	uint32_t max_nodes_pu;
	uint32_t max_submit_jobs_pu;
	uint32_t max_wall_pj;
	char *name;
	bitstr_t *preempt_bitstr;
	List preempt_list;
	uint16_t preempt_mode;
	uint32_t priority;
	assoc_mgr_qos_usage_t *usage;
	double usage_factor;
	double usage_thres;
} slurmdb_qos_rec_a;

typedef struct qos_info {
    uint32_t record_count;
    slurmdb_qos_rec_t *array;
} qos_info_t;

struct qos_info Get_qos_list() {
	struct qos_info qosinfo;

	slurmdb_qos_cond_t *qos_cond = NULL;
	void *conn = slurmdb_connection_get();
	List qoslist = slurmdb_qos_get(conn, qos_cond);

	uint16_t size = slurm_list_count(qoslist);
	qosinfo.record_count = size;
	qosinfo.array = malloc(size * sizeof(slurmdb_qos_rec_t));

	slurmdb_qos_rec_t *rec = NULL;

    //slurmdb_init_qos_rec()

	ListIterator itr = slurm_list_iterator_create(qoslist);

	int i = 0;
	while ((rec = slurm_list_next(itr))) {
  		qosinfo.array[i] = *rec;
  		i++;
	}
	slurmdb_connection_close(&conn);
	slurm_list_destroy(qoslist);

	return qosinfo;
}

 slurmdb_qos_rec_t *qos_from_list(struct qos_info *qos_rec_t, int i) {
    return (slurmdb_qos_rec_t *) &qos_rec_t->array[i];
}

slurmdb_qos_rec_t *qos_from_list_by_name(slurmdb_qos_rec_t *qos_rec_array, char* qos_name) {
	int i;
    int arraysize = sizeof(qos_rec_array);
    for (i=0; i<arraysize; i++)
    {
         if (strcmp(qos_name, (qos_rec_array[i]).name) == 0)
         {
			break;
         } else {
			return NULL;
         }
    }
	return (slurmdb_qos_rec_t *) &(qos_rec_array[i]);
}
*/
import "C"
import pbslurm "code.gitlink.org.cn/JCCE/PCM.git/adaptor/pcm_slurm/gen/idl"

func ListQoses() []*pbslurm.QosInfo {
	var goQosBuffer pbslurm.QosInfos
	cQosBuffer := C.Get_qos_list()

	goQosBuffer.RecordCount = uint32(cQosBuffer.record_count)
	for i := uint32(0); i < goQosBuffer.RecordCount; i++ {
		qos := C.qos_from_list(&cQosBuffer, C.int(i))
		goQos := QosConvertCToGo(qos)
		goQosBuffer.QosInfos = append(goQosBuffer.QosInfos, &goQos)
	}
	return goQosBuffer.QosInfos
}

func GetQos(name string) pbslurm.QosInfo {
	qos := pbslurm.QosInfo{}
	qosList := C.Get_qos_list()
	cQos := C.qos_from_list_by_name(qosList.array, C.CString(name))

	if cQos == nil {
		return qos
	}
	goQos := QosConvertCToGo(cQos)
	return goQos
}

func QosConvertCToGo(cStruct *C.slurmdb_qos_rec_t) pbslurm.QosInfo {
	var goStruct pbslurm.QosInfo

	goStruct.Description = C.GoString(cStruct.description)
	goStruct.Flags = uint32(cStruct.flags)
	goStruct.GrpCpus = uint32(cStruct.grp_cpus)
	goStruct.GraceTime = uint32(cStruct.grace_time)
	goStruct.GrpCpuMins = uint64(cStruct.grp_cpu_mins)
	goStruct.GrpCpuRunMins = uint64(cStruct.grp_cpu_run_mins)
	goStruct.GrpJobs = uint32(cStruct.grp_jobs)
	goStruct.GrpMem = uint32(cStruct.grp_mem)
	goStruct.GrpNodes = uint32(cStruct.grp_nodes)
	goStruct.GrpSubmitJobs = uint32(cStruct.grp_submit_jobs)
	goStruct.GrpWall = uint32(cStruct.grp_wall)
	goStruct.Id = uint32(cStruct.id)
	goStruct.MaxCpusPj = uint32(cStruct.max_cpus_pj)
	goStruct.MaxCpuMinsPj = uint64(cStruct.max_cpu_mins_pj)
	goStruct.MaxCpusPu = uint32(cStruct.max_cpus_pu)
	goStruct.MaxJobsPu = uint32(cStruct.max_jobs_pu)
	goStruct.MaxCpuRunMinsPu = uint64(cStruct.max_cpu_run_mins_pu)
	goStruct.MaxNodesPj = uint32(cStruct.max_nodes_pj)
	goStruct.MaxNodesPu = uint32(cStruct.max_nodes_pu)
	goStruct.MaxSubmitJobsPu = uint32(cStruct.max_submit_jobs_pu)
	goStruct.MaxWallPj = uint32(cStruct.max_wall_pj)
	goStruct.Name = C.GoString(cStruct.name)
	goStruct.Priority = uint32(cStruct.priority)
	//go_struct.PreemptList = c_struct.preempt_list
	goStruct.PreemptMode = uint32(cStruct.preempt_mode)
	goStruct.UsageThres = float64(cStruct.usage_factor)
	goStruct.UsageFactor = float64(cStruct.usage_thres)

	return goStruct
}
