package main

import (
	"code.gitlink.org.cn/JCCE/PCM.git/adaptor/pcm_pod/gen/idl"
	slurmTianhe "code.gitlink.org.cn/JCCE/PCM.git/adaptor/pcm_slurm/gen/idl"
	"code.gitlink.org.cn/JCCE/PCM.git/adaptor/pcm_vm/gen/idl"
	"code.gitlink.org.cn/JCCE/PCM.git/common/config"
	"code.gitlink.org.cn/JCCE/PCM.git/common/global"
	"context"
	"github.com/grpc-ecosystem/grpc-gateway/v2/runtime"
	"go.uber.org/zap"
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials/insecure"
	"log"
	"net/http"
)

func main() {
	// 初始化日志以及配置
	lg, err := zap.NewDevelopment()
	config.InitConfig()
	if err != nil {
		log.Fatalf("cannot create zap logger: %v", err)
	}

	c := context.Background()
	c, cancel := context.WithCancel(c)
	defer cancel()

	mux := runtime.NewServeMux()

	serverConfig := []struct {
		name         string
		addr         string
		registerFunc func(ctx context.Context, mux *runtime.ServeMux, endpoint string, opts []grpc.DialOption) (err error)
	}{
		{
			name:         "pcm_vm",
			addr:         global.S.VmInfo.Address,
			registerFunc: ecspb.RegisterEcsServiceHandlerFromEndpoint,
		},
		{
			name:         "pcm_pod",
			addr:         global.S.PodInfo.Address,
			registerFunc: podpb.RegisterPodServiceHandlerFromEndpoint,
		},
		{
			name:         "pcm_slurm",
			addr:         global.S.SlurmInfo.Address,
			registerFunc: slurmTianhe.RegisterSlurmServiceHandlerFromEndpoint,
		},
	}

	for _, s := range serverConfig {
		err := s.registerFunc(
			c, mux, s.addr,
			[]grpc.DialOption{grpc.WithTransportCredentials(insecure.NewCredentials())},
		)
		if err != nil {
			lg.Sugar().Fatalf("cannot register service %s: %v", s.name, err)
		}
	}
	lg.Sugar().Infof("grpc gateway started at %s", global.S.GatewayInfo.Address)
	lg.Sugar().Fatal(http.ListenAndServe(global.S.GatewayInfo.Address, mux))
}
